@@ -1,8 +1,8 @@
-#!/usr/local/bin/perl
+#!/usr/bin/perl
 use IO::Socket;
 use Getopt::Long;
 $|=1;
-# last update: 01.08.2002
+# last update: 01.10.2002
 
 # --------------------------------------------------------------------#
 #                               Nikto                                 #
@@ -47,10 +47,10 @@ $|=1;
 
 #######################################################################
 # global var/definitions
-use vars qw/$URI $SKIPLOOKUP $CONTENT $ITEMCOUNT $CLIOPTS $TIME $EXECDIR @PRINT @CGIDIRS @COOKIES %FILES/;
+use vars qw/$URI $SKIPLOOKUP $CONTENT $ITEMCOUNT $CLIOPTS $PAUSE $TIME @PRINT @CGIDIRS @COOKIES %FILES/;
 use vars qw/%STATS %NIKTO %AUTH %OUTPUT %ROOTS %METHD %FILES %RESPS %INFOS %SERVER %request %result %JAR %DATAS/;
 my ($HOST, $HOSTAUTH, $OKTRAP) = "";
-$NIKTO{version}="1.015";
+$NIKTO{version}="1.016";
 $NIKTO{name}="Nikto";
 my $rstring=rand(4);
 #######################################################################
@@ -61,15 +61,8 @@ my $rstring=rand(4);
 # CGI Directories - will attempt to figure out which are 'right' unless we are forced to check all
   @CGIDIRS=qw(/bin/ /cgi/ /cgi-bin/ /cgi-sys/ /cgi-local/ /htbin/ /cgibin/ /cgis/ /scripts/ /cgi-win/);
 
-# Plugin/db/lib stuff
-  # get the correct path to nikto.pl
-  $EXECDIR=$ENV{_};
-  chomp($EXECDIR);
-  $EXECDIR =~ s/nikto.pl$//;
-  if ($EXECDIR =~ /(perl|perl\.exe)$/) { $EXECDIR=""; }  # executed as 'perl nikto.pl' ...
-  $NIKTO{plugindir}="$EXECDIR\plugins";
-  $FILES{dbfile}="$NIKTO{plugindir}/scan_database.db";
-  require "$NIKTO{plugindir}/libwhisker.pm";
+  &load_confs;
+
   lw::http_init_request(\%request);
 
 # this stuff should all move to CLI options or config file...
@@ -82,7 +75,7 @@ my $rstring=rand(4);
 if ($ARGV[0] eq "dbcheck") { &dbcheck; }   # just check the scan_database.db for duplicates
 elsif ($ARGV[0] eq "update") { &check_updates; }  
 
-$NIKTO{suppressauth}=$OUTPUT{html}=$OUTPUT{verbose}=$SKIPLOOKUP=$STATS{moved}=$STATS{okay}=$STATS{requests}=$ITEMCOUNT=0;
+$PAUSE=$NIKTO{suppressauth}=$OUTPUT{html}=$OUTPUT{verbose}=$SKIPLOOKUP=$STATS{moved}=$STATS{okay}=$STATS{requests}=$ITEMCOUNT=0;
 $OKTRAP=1;
 my $STARTTIME= localtime();
 $CLIOPTS="
@@ -118,6 +111,7 @@ GetOptions("verbose"  => \$OUTPUT{verbose},
            "id=s"     => \$HOSTAUTH,
            "port=s"   => \$SERVER{port},
            "timeout=s"=> \$TIME,
+           "x=s"      => \$PAUSE,
            "host=s"   => \$HOST);
 #######################################################################
 # check CLI options & setup host info
@@ -206,9 +200,8 @@ if ($HOSTAUTH ne "")
 
  # connection details
 if ($OUTPUT{debug}) {
- dprint("- Initial Connection Details:\n");
- foreach my $item (sort keys %result) { if ($item eq "whisker") { next; } dprint("- $item \t$result{$item}\n"); }
- foreach my $item (sort keys %{$result{'whisker'}}) { dprint("- $item \t$result{'whisker'}->{$item}\n"); }
+ dprint("- Initial Server Connect Details:\n");
+ &dump_result_hash;
 }
 
  if ($SERVER{servertype} ne "") { fprint("+ Server: $SERVER{servertype}\n"); }
@@ -231,10 +224,11 @@ if (!$SERVER{forcecgi}) { @CGIDIRS=&check_cgi; }
 # this is the looped code for all the checks
 #######################################################################
  for (my $CHECKID=1;$CHECKID<=$ITEMCOUNT;$CHECKID++)
-  {
+  { 
    $URI = "$ROOTS{$CHECKID}$FILES{$CHECKID}";
    if ($URI eq "//") { $URI="/"; } # right???
    (my $RES, $CONTENT) = fetch($URI,$METHD{$CHECKID},$DATAS{$CHECKID});
+   if ($RES eq "") { &dump_request_hash; &dump_result_hash; }
    vprint("- $RES for $METHD{$CHECKID}:\t$URI\n");
 
    if ($RESPS{$CHECKID} =~ /[^0-9]/)   # response has text to match--before response code, just in case
@@ -279,6 +273,7 @@ if (!$SERVER{forcecgi}) { @CGIDIRS=&check_cgi; }
   }
 
  # end check loop
+ sleep $PAUSE;
  }
 
 #######################################################################
@@ -295,6 +290,9 @@ fprint("- $ITEMCOUNT items checked on remote host\n");
 #######################################################################
 # output to file?
 my $linkhost=$SERVER{hostname} || $SERVER{ip};
+my $httpstring="http";
+if ($SERVER{ssl}) { $httpstring="https"; }
+ 
 if ($OUTPUT{file} ne "")
   {
    open(OUT,">>$OUTPUT{file}") || die print "ERROR: Unable to open '$OUTPUT{file}' for write: $!\n";
@@ -318,12 +316,12 @@ if ($OUTPUT{file} ne "")
      { 
       $line .= "<br>"; 
       if ($line =~ /---------/) { $line="<hr>"; }
-      elsif ($line =~ /$NIKTO{name} v/)   { $line=~s/$NIKTO{name}/\<a href=\"http:\/\/www.cirt.net\/\">$NIKTO{name}\<\/a\>/;
+      elsif ($line =~ /$NIKTO{name} v/)   { $line=~s/$NIKTO{name}/\<a href=\"$httpstring\:\/\/www.cirt.net\/\">$NIKTO{name}\<\/a\>/;
                                      $line ="<center><font size=+1>$line</font></center>\n"; }
       elsif ($line =~ /^(\+|\-) Target IP/)  
-        { $line =~ s/$SERVER{ip}/<a href=\"http:\/\/$SERVER{ip}:$SERVER{port}\/\">$SERVER{ip}<\/a>/; }
+        { $line =~ s/$SERVER{ip}/<a href=\"$httpstring\:\/\/$SERVER{ip}:$SERVER{port}\/\">$SERVER{ip}<\/a>/; }
       elsif ($line =~ /^(\+|\-) Target Host/)  
-        { $line =~ s/$SERVER{hostname}/<a href=\"http:\/\/$SERVER{hostname}:$SERVER{port}\/\">$SERVER{hostname}<\/a>/; }
+        { $line =~ s/$SERVER{hostname}/<a href=\"$httpstring\:\/\/$SERVER{hostname}:$SERVER{port}\/\">$SERVER{hostname}<\/a>/; }
       elsif ($line =~ /^\+ \//) # item
        {
        if ($line =~ /\(GET\)/) {
@@ -334,10 +332,10 @@ if ($OUTPUT{file} ne "")
         $disp =~ s/\</\&lt\;/g;
         $disp =~ s/\>/\&gt\;/g;
         $TEMP[1] =~ s/([^a-zA-Z0-9\s])/\\$1/g;
-        $line =~ s/$TEMP[1]/<a href=\"http:\/\/$linkhost:$SERVER{port}\/$r\">$disp<\/a>/;
+        $line =~ s/$TEMP[1]/<a href=\"$httpstring\:\/\/$linkhost:$SERVER{port}\/$r\">$disp<\/a>/;
         }
         # make a link for CVE identifiers, like CVE-1999-0346 or CAN-1999-0346
-        $line =~ s/((cve|can)\-[0-9]{4}-[0-9]{4})/<a href=\"http:\/\/icat.nist.gov\/icat.cfm\?cvename=$1\">$1<\/a>/gi;
+        $line =~ s/((cve|can)\-[0-9]{4}-[0-9]{4})/<a href=\"$httpstring\:\/\/icat.nist.gov\/icat.cfm\?cvename=$1\">$1<\/a>/gi;
        }
      $line =~ s/^\- /<li>/;
      $line =~ s/^\+ /<li>/;
@@ -353,7 +351,36 @@ exit;
 #################################################################################
 # FUNCTIONS
 #################################################################################
+sub load_confs
+{
+# Plugin/db/lib stuff
+  # get the correct path to nikto.pl
+  # try pwd?
+  my $NIKTODIR="";
+  if (-d "$ENV{PWD}/plugins") { $NIKTODIR="$ENV{PWD}/"; }
+  elsif (-d "plugins") { $NIKTODIR=""; }
+  else
+  {
+   my $EXECDIR=$ENV{_};
+   chomp($EXECDIR);
+   $EXECDIR =~ s/nikto.pl$//;
+   if ($EXECDIR =~ /(perl|perl\.exe)$/) { $EXECDIR=""; }  # executed as 'perl nikto.pl' ...
+   if (-e "$EXECDIR/plugins") { $NIKTODIR="$EXECDIR/"; }
+  }
+  $NIKTO{plugindir}="$NIKTODIR\plugins";
 
+  if (!(-d $NIKTO{plugindir}))
+  {
+   print "I can't find 'plugins' directory. ";
+   print "I looked in \& around:\n\t$ENV{PWD}\n\t$ENV{_}\n";
+   print "Try switching to the 'nikto' directory so that the plugins dir can be found.\n";
+   exit;
+  }
+
+  $FILES{dbfile}="$NIKTO{plugindir}/scan_database.db";
+  require "$NIKTO{plugindir}/libwhisker.pm";
+ return;
+}
 #################################################################################
 # run_plugins
 # load plugins  & run them
@@ -648,6 +675,23 @@ sub dbcheck {
  exit;
 }
 #######################################################################
+# spit out all the details
+#######################################################################
+sub dump_result_hash
+{
+ foreach my $item (sort keys %result) { if ($item eq "whisker") { next; } dprint("- $item \t$result{$item}\n"); }
+ foreach my $item (sort keys %{$result{'whisker'}}) { dprint("- $item \t$result{'whisker'}->{$item}\n"); }
+}
+#######################################################################
+#######################################################################
+# spit out all the details
+#######################################################################
+sub dump_request_hash
+{
+ foreach my $item (sort keys %request) { if ($item eq "whisker") { next; } dprint("- $item \t$request{$item}\n"); }
+ foreach my $item (sort keys %{$request{'whisker'}}) { dprint("- $item \t$request{'whisker'}->{$item}\n"); }
+}
+#######################################################################
 # check_responses
 # check what the 200/404 messages are...
 #######################################################################
@@ -744,6 +788,7 @@ sub check_cgi
 sub fetch
 {
  my $uri=$_[0] || return;
+ &lw::http_reset;
  if ($uri eq "//") { $uri="/"; } # trap for some weird ones
  $request{'whisker'}->{'method'} = $_[1] || "GET";
  $request{'whisker'}->{'uri'}    = $uri;
@@ -753,7 +798,6 @@ sub fetch
  lw::http_do_request(\%request,\%result);
  
  if ($result{'Set-Cookie'} ne "") { push(@COOKIES,"/--=--$result{'Set-Cookie'}"); }
- &lw::http_reset;
 
  return $result{'whisker'}->{'http_resp'}, $result{'whisker'}->{'data'};
 }
